@@ -8,7 +8,7 @@ from math import isnan
 import random
 
 from unittest.mock import Mock, MagicMock, patch
-from scipy.sparse import csr_matrix, issparse
+import scipy.sparse as sp
 import numpy as np
 
 from Orange import data
@@ -649,6 +649,19 @@ class TableTestCase(unittest.TestCase):
         self.assertFalse(np.all(t.Y == copy.Y))
         self.assertFalse(np.all(t.metas == copy.metas))
 
+    def test_copy_sparse(self):
+        t = data.Table('iris')
+        t.X = sp.csr_matrix(t.X)
+        copy = t.copy()
+
+        self.assertEqual((t.X != copy.X).nnz, 0)      # sparse matrices match by content
+        np.testing.assert_equal(t.Y, copy.Y)
+        np.testing.assert_equal(t.metas, copy.metas)
+
+        self.assertNotEqual(id(t.X), id(copy.X))
+        self.assertNotEqual(id(t._Y), id(copy._Y))
+        self.assertNotEqual(id(t.metas), id(copy.metas))
+
     def test_concatenate(self):
         d1 = data.Domain([data.ContinuousVariable('a1')])
         t1 = data.Table.from_numpy(d1, [[1],
@@ -1009,6 +1022,12 @@ class TableTestCase(unittest.TestCase):
         x = filter.Values([f])(d)
         self.assertEqual(len(x), 7)
 
+    def test_valueFilter_discrete_meta_is_defined(self):
+        d = data.Table("test9.tab")
+        f = filter.FilterDiscrete(-4, None)
+        x = filter.Values([f])(d)
+        self.assertEqual(len(x), 8)
+
     def test_valueFilter_string_case_sens(self):
         d = data.Table("zoo")
         col = d[:, "name"].metas[:, 0]
@@ -1300,8 +1319,6 @@ class CreateTableWithUrl(TableTests):
     class _MockUrlOpen(MagicMock):
         headers = {'content-disposition': 'attachment; filename="Something-FormResponses.tsv"; '
                                           'filename*=UTF-8''Something%20%28Responses%29.tsv'}
-        url = 'https://docs.google.com/spreadsheets/d/ABCD/edit'
-
         def __enter__(self): return self
 
         def __exit__(self, *args, **kwargs): pass
@@ -1314,12 +1331,16 @@ a\tb\tc
     urlopen = _MockUrlOpen()
 
     @patch('Orange.data.io.urlopen', urlopen)
-    def test_google_sheets(self):
-        d = data.Table(self.urlopen.url)
-        self.urlopen.assert_called_with('https://docs.google.com/spreadsheets/d/ABCD/export?format=tsv',
-                                        timeout=10)
-        self.assertEqual(len(d), 2)
-        self.assertEqual(d.name, 'Something-FormResponses')
+    def test_trimmed_urls(self):
+        for url in ('https://docs.google.com/spreadsheets/d/ABCD/edit',
+                    'https://www.dropbox.com/s/ABCD/filename.csv'):
+            self._MockUrlOpen.url = url
+            d = data.Table(url)
+            request = self.urlopen.call_args[0][0]
+            self.assertNotEqual(url, request.full_url)
+            self.assertIn('Mozilla/5.0', request.headers.get('User-agent', ''))
+            self.assertEqual(len(d), 2)
+            self.assertEqual(d.name, 'Something-FormResponses')
 
 
 class CreateTableWithDomain(TableTests):
@@ -1734,23 +1755,68 @@ class CreateTableWithDomainAndTable(TableTests):
         self.assert_table_with_filter_matches(
             new_table, self.table[:0], xcols=order, ycols=order, mcols=order)
 
-    def test_from_table_on_sparse_data(self):
+    def test_from_table_sparse_move_some_to_empty_metas(self):
         iris = data.Table("iris")
-        iris.X = csr_matrix(iris.X)
-
+        iris.X = sp.csr_matrix(iris.X)
         new_domain = data.domain.Domain(iris.domain.attributes[:2], iris.domain.class_vars,
-                                        iris.domain.metas, source=iris.domain)
+                                        iris.domain.attributes[2:], source=iris.domain)
         new_iris = data.Table.from_table(new_domain, iris)
-        self.assertTrue(issparse(new_iris.X))
-        self.assertEqual(new_iris.X.shape[1], 2)
-        self.assertEqual(len(new_iris.domain.attributes), 2)
 
-        all_vars = chain(iris.domain.variables, iris.domain.metas)
-        n_all = len(iris.domain) + len(iris.domain.metas)
-        new_domain = data.domain.Domain([], [], all_vars, source=iris.domain)
+        self.assertTrue(sp.issparse(new_iris.X))
+        self.assertTrue(sp.issparse(new_iris.metas))
+        self.assertEqual(new_iris.X.shape, (len(iris), 2))
+        self.assertEqual(new_iris.metas.shape, (len(iris), 2))
+
+        # move back
+        back_iris = data.Table.from_table(iris.domain, new_iris)
+        self.assertEqual(back_iris.domain, iris.domain)
+        self.assertTrue(sp.issparse(back_iris.X))
+        self.assertTrue(sp.issparse(back_iris.metas))
+        self.assertEqual(back_iris.X.shape, iris.X.shape)
+        self.assertEqual(back_iris.metas.shape, iris.metas.shape)
+
+    def test_from_table_sparse_move_all_to_empty_metas(self):
+        iris = data.Table("iris")
+        iris.X = sp.csr_matrix(iris.X)
+        new_domain = data.domain.Domain([], iris.domain.class_vars,
+                                        iris.domain.attributes, source=iris.domain)
         new_iris = data.Table.from_table(new_domain, iris)
-        self.assertEqual(len(new_iris.domain.metas), n_all)
-        self.assertEqual(new_iris.metas.shape[1], n_all)
+        
+        self.assertTrue(sp.issparse(new_iris.X))
+        self.assertTrue(sp.issparse(new_iris.metas))
+        self.assertEqual(new_iris.X.shape, (len(iris), 0))
+        self.assertEqual(new_iris.metas.shape, (len(iris), 4))
+
+        # move back
+        back_iris = data.Table.from_table(iris.domain, new_iris)
+        self.assertEqual(back_iris.domain, iris.domain)
+        self.assertTrue(sp.issparse(back_iris.X))
+        self.assertTrue(sp.issparse(back_iris.metas))
+        self.assertEqual(back_iris.X.shape, iris.X.shape)
+        self.assertEqual(back_iris.metas.shape, iris.metas.shape)
+
+    def test_from_table_sparse_move_to_nonempty_metas(self):
+        brown = data.Table("brown-selected")
+        brown.X = sp.csr_matrix(brown.X)
+        n_attr = len(brown.domain.attributes)
+        n_metas = len(brown.domain.metas)
+        new_domain = data.domain.Domain(brown.domain.attributes[:-10], brown.domain.class_vars,
+                                        brown.domain.attributes[-10:] + brown.domain.metas,
+                                        source=brown.domain)
+        new_brown = data.Table.from_table(new_domain, brown)
+
+        self.assertTrue(sp.issparse(new_brown.X))
+        self.assertFalse(sp.issparse(new_brown.metas))
+        self.assertEqual(new_brown.X.shape, (len(new_brown), n_attr-10))
+        self.assertEqual(new_brown.metas.shape, (len(new_brown), n_metas+10))
+
+        # move back
+        back_brown = data.Table.from_table(brown.domain, new_brown)
+        self.assertEqual(brown.domain, back_brown.domain)
+        self.assertTrue(sp.issparse(back_brown.X))
+        self.assertFalse(sp.issparse(back_brown.metas))
+        self.assertEqual(back_brown.X.shape, brown.X.shape)
+        self.assertEqual(back_brown.metas.shape, brown.metas.shape)
 
     def assert_table_with_filter_matches(
             self, new_table, old_table,
