@@ -24,6 +24,8 @@ class Place:
 
 
 class VarTableModel(QtCore.QAbstractTableModel):
+    DISCRETE_VALUE_DISPLAY_LIMIT = 20
+
     places = "feature", "target", "meta", "skip"
     typenames = "nominal", "numeric", "string", "datetime"
     vartypes = DiscreteVariable, ContinuousVariable, StringVariable, TimeVariable
@@ -46,15 +48,24 @@ class VarTableModel(QtCore.QAbstractTableModel):
                     return False
             return False
 
+        def discrete_value_display(value_list):
+            result = ", ".join(str(v) for v in value_list[:VarTableModel.DISCRETE_VALUE_DISPLAY_LIMIT])
+            if len(value_list) > VarTableModel.DISCRETE_VALUE_DISPLAY_LIMIT:
+                result += ", ..."
+            return result
+
         self.modelAboutToBeReset.emit()
-        self.variables[:] = self.original = [
-            [var.name, type(var), place,
-             ", ".join(var.values) if var.is_discrete else "",
-             may_be_numeric(var)]
-            for place, vars in enumerate(
-                (domain.attributes, domain.class_vars, domain.metas))
-            for var in vars
-        ]
+        if domain is None:
+            self.variables.clear()
+        else:
+            self.variables[:] = self.original = [
+                [var.name, type(var), place,
+                 discrete_value_display(var.values) if var.is_discrete else "",
+                 may_be_numeric(var)]
+                for place, vars in enumerate(
+                    (domain.attributes, domain.class_vars, domain.metas))
+                for var in vars
+            ]
         self.modelReset.emit()
 
     def reset(self):
